package com.caimei.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.caimei.model.ResponseJson;
import com.caimei.service.OrderSubmitService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2020/3/21
 */
@Api(tags = "订单结算")
@Slf4j
@RestController
@RequestMapping("/order")
public class OrderSubmitApi {

    private OrderSubmitService orderSubmitService;

    @Autowired
    public void setOrderSubmitService(OrderSubmitService orderSubmitService) {
        this.orderSubmitService = orderSubmitService;
    }

    /**
     * 商品结算（普通商品和二手商品通用接口）
     * <p>
     * 返回参数：shopList：按照供应商分类的商品信息
     * invoice：发票信息(目前你应该用不到，适用crm端)
     * userMoney：用户余额
     * postage：邮费（-1到付，0包邮，大于0具体金额）
     */
    @ApiOperation("商品结算")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "productIds", value = "组织商品ID串用逗号隔开。", required = true),
            @ApiImplicitParam(name = "count", value = "立即购买商品数量，购物车count为null。"),
            @ApiImplicitParam(name = "userId", value = "机构用户id。", required = true)
    })
    @GetMapping("/confirm")
    public ResponseJson<Map<String, Object>> orderConfirm(String productIds, Integer count, Integer userId) {
        return orderSubmitService.orderConfirm(productIds, count, userId);
    }

    /**
     * 提交订单
     *
     * @param params 参数格式：{
     *               *   "cartType":3,               //购买类型:（1购物车提交[对应表cm_cart]，2直接购买提交, 3协销下单）
     *               *   "orderSource": 2,           //订单来源：1WWW、2CRM、3APP[历史数据]、4客服[适用后台下单]、5外单[适用后台下单]、6小程序[采美,星范等]
     *               *   "serviceProviderId": 1378,  //协销ID（小程序忽略）
     *               *   "clubUserId": 10708,        //机构用户ID
     *               *   "addressId": 2732,          //地址ID
     *               *   "orderInfo": [
     *               *                   { "shopId":1001,    // 供应商ID
     *               *                     "note":备注,
     *               *                     "productInfo":[   // 商品id，数量，赠品数,商品类型
     *               *                         {"productId": 2789, "productNum": 1, "presentNum":0,"productType":2},
     *               *                         {"productId": 2789, "productNum": 1, "presentNum":0,"productType":0}
     *               *                      ]
     *               *                   },
     *               *                   {多个供应商商品数据结构同上}
     *               *                ]
     *               *
     *               *   "payInfo": {               // 订单信息
     *               *       "orderShouldPayFee": 609.11,
     *               *       "balancePayFlag": 0,
     *               *       "clauseId": "2",
     *               *       "freight": "15",
     *               *       "freePostFlag": 1,
     *               *       "rebateFlag":0
     *               *   },
     *               *   "orderInvoice": {}// 发票信息： {"type": 0},// 不开发票
     *               *                   {
     *               *                       "type": 1,             // 普通发票
     *               *                       "invoiceContent": "明细 ",
     *               *                       "invoiceTitleType": 1,
     *               *                       "corporationTaxNum": "XXX4156465465",
     *               *                       "invoiceTitle": "企业抬头"
     *               *                   },
     *               *                   {
     *               *                       "type": 2,             // 增值税发票
     *               *                       "invoiceTitle": "单位名称",
     *               *                       "corporationTaxNum": "NSRSBM97897",
     *               *                       "registeredAddress": "注册地址",
     *               *                       "registeredPhone": "15814011616",
     *               *                       "openBank": "开户银行",
     *               *                       "bankAccountNo": "987987465465464"
     *               *                   }
     *               *}
     * @return code: -1=(用户账户异常,数据异常,操作异常等)，
     * 1提交成功（支付完成）,
     * 2提交成功(未支付)，
     * code为1和2的情况下出现 data{orderNo:订单号，orderID:订单ID,payTotalFee:订单金额，orderMark：订单标识}
     */
    @ApiOperation("提交订单")
    @PostMapping("/submit")
    public ResponseJson<Map<String, String>> orderSubmit(@RequestBody String params) {
        if (StringUtils.isBlank(params)) {
            return ResponseJson.error("数据异常", null);
        }
        Integer cartType = null;
        String orderSource = null;
        Integer clubUserId = null;
        Integer addressId = null;
        List<Map<String, Object>> orderInfo = null;
        Map<String, Object> payInfo = null;
        Map<String, Object> orderInvoice = null;
        // 校验传入参数的正确性
        try {
            JSONObject paramsMap = JSON.parseObject(params);
            if (null == paramsMap) {
                return ResponseJson.error("数据异常,参数不能为空", null);
            }
            orderSource = paramsMap.getString("orderSource");
            if (StringUtils.isEmpty(orderSource)) {
                return ResponseJson.error("订单来源异常", null);
            }
            cartType = paramsMap.getInteger("cartType");
            if (null == cartType) {
                return ResponseJson.error("购买类型异常", null);
            }
            clubUserId = paramsMap.getInteger("clubUserId");
            if (null == clubUserId) {
                return ResponseJson.error("机构用户Id异常", null);
            }
            addressId = paramsMap.getInteger("addressId");
            if (null == addressId) {
                return ResponseJson.error("地址数据异常", null);
            }
            orderInfo = (List<Map<String, Object>>) paramsMap.get("orderInfo");
            if (null == orderInfo) {
                return ResponseJson.error("订单商品数据异常", null);
            }
            payInfo = paramsMap.getJSONObject("payInfo");
            if (null == payInfo) {
                return ResponseJson.error("订单金额数据异常", null);
            }
            orderInvoice = paramsMap.getJSONObject("orderInvoice");
            if (null == orderInvoice) {
                return ResponseJson.error("订单发票数据异常", null);
            }
        } catch (Exception e) {
            log.error("<<<<< 系统异常 >>>>>" + e);
            return ResponseJson.error("数据异常", null);
        }

        log.info("****** 提交订单参数：【机构自己下单】：" + params);
        return orderSubmitService.orderSubmit(cartType, orderSource, clubUserId, addressId, orderInfo, payInfo, orderInvoice);
    }
}
